{
   muu TrueColor library for TMT Pascal - Burn example
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   Credits go to Frank Jan Sorensen alias Frank Patxi (fjs@lab.jt.dk)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

USES mArea,mClip,mFormat,mSurface,mConsole,mTarga,mc_VESA,mc_Wrap,Nova,memory,CRT,Error,HTimer;

// {$DEFINE debug_wrapper}  // enable this ONLY for debugging

TYPE Pal8 = ARRAY[BYTE,0..2] OF byte;

CONST SCREEN_WIDTH      = 320;
      SCREEN_HEIGHT     = 200;

        // BURN constants
      BURN_ROOT_RAND    = 20;
      BURN_DECAY        = 10;
      BURN_YSTART       = SCREEN_HEIGHT-100;
      BURN_SMOOTH       = 1;
      BURN_MINFIRE      = 50;
      BURN_XSTART       = 0;
      BURN_XEND         = SCREEN_WIDTH;
      BURN_WIDTH        = BURN_XEND-BURN_XSTART;
      BURN_MAXCOLOR     = 110;
      BURN_FIREINCREASE = 40;
      BURN_MOREFIRE     = 1;

    // our Surface and Console, clear them just for neatness
VAR flames:     Surface := NIL;
    screen:     Console := NIL;
    blurred:    Surface := NIL;
    flame_area: Area := Area(BURN_XSTART,BURN_YSTART,BURN_XEND,SCREEN_HEIGHT);
    dimer:      Timer := NIL;
    frames:     dword := 0;

    // set textmode
PROCEDURE set_textmode; code;
ASM
  mov  ax,  $03
  int  $10
  ret
END;

    // blurring procedure w/clipping
PROCEDURE blur8(m_source:Surface;m_dest:Surface;m_area:Area);
  VAR s_data,d_data:        pointer;
      s_next:               dword;
      s_line,d_line:        dword;
      put_width,put_height: dword;
      src_start,dest_start: dword;
BEGIN
    // clip area with both surfaces
  m_area := Clip(m_area,Area(1,1,m_source^.Width-1,m_source^.Height-1));
  m_area := Clip(m_area,Area(1,1,m_dest^.Width-1,m_dest^.Height-1));

    // determine how much to put
  put_width := m_area.Right-m_area.Left;
  put_height := m_area.Bottom-m_area.Top;

  IF (put_width <= 0) OR
     (put_height <= 0) THEN
       Exit;

    // data start positions
  s_data := ptr(ofs(m_source^.Data^)+(m_area.Left+m_area.Top*m_source^.Width));
  d_data := ptr(ofs(m_dest^.Data^)+(m_area.Left+m_area.Top*m_dest^.Width));

    // next line
  s_line := m_source^.Width-put_width;
  d_line := m_dest^.Width-put_width;

    // one line up/down
  s_next := m_source^.Width;
  
  ASM
    mov  esi, s_data
    mov  edi, d_data

  @@yloop:
    mov  ecx, put_width
    xor  ebx, ebx

  @@xloop:
    mov  edx, s_next

        // right
    mov  al,  [esi+1]
    mov  bx,  ax
        // left
    mov  al,  [esi-1]
    add  bx,  ax
        // above
    mov  al,  [esi+edx*1]
    add  bx,  ax
        // below
    xor  edx, -1
    inc  edx
    mov  al,  [esi+edx*1]
    add  bx,  ax

    shr  bx,  2
    mov  [edi],  bl

    inc  esi
    inc  edi

    dec  ecx
    jnz  @@xloop

    add  esi, s_line
    add  edi, d_line

    dec  put_height
    jnz  @@yloop
  END;    
END;

    // returns a random number between [-r] and [r]
FUNCTION rand(r:longint): longint;
  result := random(r*2+1)-r;


begin
    // We will use the VBEConsole to display the image, therefore we overload
    // the screen type when creating it (creating means allocating)
    // (if debug_wrapper is defined, wrap VBEConsole to wrapper
  {$IFNDEF debug_wrapper}
    new(VBEConsole(screen),Create);
  {$ELSE}
    new(TTYConsole(screen),Create);
    DECLARE VAR deb: Console;
    BEGIN
      new(VBEConsole(deb),Create);
      screen^.Wrap(deb);
    END;
  {$ENDIF}

    // read options into Console, which will be used instead of the default
    // values if specified (here we load options from "mtc.cfg")
  screen^.getOptions(mtc_option_file);

    // options are set, and we can safely proceed opening our Console. Here
    // it's called "Image example", which is a suiting name:)
  screen^.Open('Image example',SCREEN_WIDTH,SCREEN_HEIGHT,Format(unknown));

    // check if an error occurred when opening, and exit with error if so
  IF (screen^.Error) THEN
    fail('Failed to create console');

    // prepare VPAL surface and load palette
  flames := Surface(SCREEN_WIDTH,SCREEN_HEIGHT+1,Format(VPAL));

  DECLARE VAR f:    file;
              t:    Pal8;
              c:    dword;
  BEGIN
    assign(f,'data\burn.pal');
    reset(f,1);
    blockread(f,t,768);
    close(f);
    FOR c := 0 TO 255 DO
    BEGIN
      flames^.sFormat.Palette[c,VBlue] := t[c,2] shl 2;
      flames^.sFormat.Palette[c,VGreen] := t[c,1] shl 2;
      flames^.sFormat.Palette[c,VRed] := t[c,0] shl 2;
    END;
  END;

  blurred := Surface(flames);

  new(dimer,Create);

    // clear keyboard buffer
  WHILE keypressed DO readkey;

  randomize;

    // start a fire immediately
  memset(ptr(ofs(flames^.Data^)+SCREEN_WIDTH*SCREEN_HEIGHT+BURN_XSTART+random(BURN_WIDTH-5)),5,$FFFFFFFF);

  dimer^.Start;
    // the fire loop
  DECLARE VAR flame_root:   pointer := ptr(ofs(flames^.Data^)+SCREEN_WIDTH*SCREEN_HEIGHT);
              x,y:          longint := 0;
              v:            longint := 0;
  REPEAT

        // this loop makes the actual flames

    FOR x := BURN_XSTART TO BURN_XEND DO
      FOR y := BURN_YSTART TO SCREEN_HEIGHT DO
      BEGIN
        v := mem[ofs(flames^.Data^)+x+y*flames^.Width];

        IF (v < BURN_DECAY) OR
           (x = BURN_XSTART) OR
           (x = BURN_XEND) THEN
          mem[ofs(flames^.Data^)+x+(y-1)*flames^.Width] := 0
        ELSE IF (v > 0) THEN
          mem[ofs(flames^.Data^)+x-(random(3)-1)+(y-1)*flames^.Width] := v-random(BURN_DECAY);
      END;

        // start a fire
    IF ( random(150) = 0) THEN
      memset(ptr(ofs(flame_root^)+BURN_XSTART+random(BURN_WIDTH-5)),5,$E4A7C4E1);

        // This loop controls the "root" of the flames
    FOR x := BURN_XSTART TO BURN_XEND DO
    BEGIN
      v := mem[ofs(flame_root^)+x];

      IF (v < BURN_MINFIRE) THEN
      BEGIN
          IF (v > 10) THEN
            v +:= random(BURN_FIREINCREASE);
      END ELSE
        v +:= rand(BURN_ROOT_RAND)+BURN_MOREFIRE;

      IF (v > 255) THEN
        v := 255;

      mem[ofs(flame_root^)+x] := v;
    END;

        // Pour water on the sides
    FOR x := 1 TO (BURN_WIDTH shr 3) DO
    BEGIN
      v := trunc(sqr(random)*(BURN_WIDTH shr 5));
      mem[ofs(flame_root^)+BURN_XSTART+v] := 0;
      mem[ofs(flame_root^)+BURN_XEND-v] := 0;
    END;

        // Smoothen the values of FrameArray to avoid "descrete" flames
    FOR x := (BURN_XSTART+BURN_SMOOTH) TO (BURN_XEND-BURN_SMOOTH) DO
    BEGIN
      v := 0;
      FOR y := -BURN_SMOOTH TO BURN_SMOOTH DO
        v +:= mem[ofs(flame_root^)+x+y];

      mem[ofs(flame_root^)+x] := v div (2*BURN_SMOOTH+1);
    END;

        // blurr the flames for a nicer effect
//    blur8(flames,blurred,Area(flame_area));

        // write to screen
//    screen^.Copy(0,BURN_YSTART,blurred,Area(flame_area));
    screen^.Copy(0,BURN_YSTART,flames,Area(flame_area));
    screen^.Update;
    frames +:= 1;

  UNTIL (keypressed);
  dimer^.Stop;

    // free memory used by the surfaces
  dispose(flames,Close);
  dispose(blurred,Close);

    // close and free Console
  screen^.Close;
  dispose(screen,Destroy);

    // set textmode, and exit
  set_textmode;

  writeln('Frames per second: ', frames/(dimer^.Elapsed/1000):0:3);writeln;

  writeln('muu TrueColor library for TMT Pascal - Burn example');
  writeln('Copyright (C) 1999 Tor Arne Gustad (anden@online.no)');
end.
