{
   muu TrueColor library for TMT Pascal - Font example
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

USES mArea,mClip,mFormat,mSurface,mConsole,mTarga,mc_VESA,mc_Wrap,Nova,mFont,CRT,HTimer,Error,Memory;

// {$DEFINE debug_wrapper}  // enable this ONLY for debugging

CONST text_next_line = 18;

    // our Console, clear the variables for neatness
VAR testconsole:    Console := NIL;
    testsurface:    Surface := NIL;
    testfont:       muuFont := NIL;
    frames:         dword := 0;
    texting:        text;
    textline:       string := '';
    texttimer:      Timer := NIL;
    textypos:       dword := 0;
    w:              boolean := true;


    // set textmode
PROCEDURE set_textmode; assembler;
ASM
  mov  ax,  $03
  int  $10
END;

    // blurring procedure w/clipping
PROCEDURE blurr32(m_source:Surface;m_dest:Surface;m_area:Area);
  VAR s_data,d_data:        pointer;
      s_next:               dword := 0;
      s_line,d_line:        dword := 0;
      put_width,put_height: dword := 0;
      line_position:        dword := 0;
BEGIN
    // clip area with both surfaces
  m_area := Clip(m_area,Area(1,1,m_source^.Width-1,m_source^.Height-1));
  m_area := Clip(m_area,Area(1,1,m_dest^.Width-1,m_dest^.Height-1));

    // determine how much to put
  put_width := (m_area.Right-m_area.Left);
  put_height := (m_area.Bottom-m_area.Top);

  IF (put_width <= 0) OR
     (put_height <= 0) THEN
       Exit;

    // data start positions
  s_data := ptr(ofs(m_source^.Data^)+(m_area.Left+m_area.Top*m_source^.Width) shl 2);
  d_data := ptr(ofs(m_dest^.Data^)+(m_area.Left+m_area.Top*m_dest^.Width) shl 2);

    // next line
  s_line := (m_source^.Width-put_width) shl 2;
  d_line := (m_dest^.Width-put_width) shl 2;

    // one line up/down
  s_next := m_source^.Width shl 2;

  ASM
    mov  esi, s_data
    mov  edi, d_data

  @@yloop:
    mov  eax, put_width
    mov  line_position, eax

    xor  ebx, ebx

  @@xloop:
    mov  ecx, 3
    mov  edx, s_next

    // loop through B,G,R and A
  @@pixloop:
    mov  al,  [esi+4]
    mov  bx,  ax

    mov  al,  [esi-4]
    add  bx,  ax

    shr  bx,  1
    mov  [edi],  bl

    inc  esi
    inc  edi

    dec  ecx
    jnz  @@pixloop

    inc  esi
    inc  edi    

    dec  line_position
    jnz  @@xloop

    add  esi, s_line
    add  edi, d_line

    dec  put_height
    jnz  @@yloop
  END;    
END;


        // -------------- START OF MAIN CODE ----------------

begin
    // We will use the VBEConsole to display the image, therefore we overload
    // the testconsole type when creating it (creating means allocating)
    // (if debug_wrapper is defined, wrap VBEConsole to wrapper
  {$IFNDEF debug_wrapper}
    new(VBEConsole(testconsole),Create);
  {$ELSE}
    new(TTYConsole(testconsole),Create);
    DECLARE VAR deb: Console;
    BEGIN
      new(VBEConsole(deb),Create);
      testconsole^.Wrap(deb);
    END;
  {$ENDIF}

    // read options into Console, which will be used instead of the default
    // values if specified (here we load options from "mtc.cfg")
  testconsole^.getOptions(mtc_option_file);

    // options are set, and we can safely proceed opening our Console.
  testconsole^.Open('Font example',0,0,Format(Unknown));

    // check if an error occurred when opening, and exit with error if so
  IF (testconsole^.Error) THEN
    fail('Failed to create console');

    // load font data
  new(testfont,Open('data\matrix.dfs'));

    // load texting
  assign(texting,'data\font.txt');
  {$I-}reset(texting);{$I+}
  IF (IOResult <> 0) THEN
    fail('Failed to load `data\font.txt''');

  new(texttimer,Create);

    // create Surface and write a string to it
  testsurface := Surface(testconsole^.Info.Width,testconsole^.Info.Height,Format(BGR8888));

  texttimer^.Start;
  textypos := 0;
  REPEAT
    IF (texttimer^.Elapsed >= 800) THEN
    BEGIN
      IF (eof(texting)) THEN
      BEGIN
        close(texting);
        reset(texting);
        textypos := 0;
        textline := '';
      END ELSE BEGIN
        textypos +:= text_next_line;
        readln(texting,textline);
      END;
      testfont^.Write(10,textypos,testsurface,textline,mfAdd);
      texttimer^.Reset;
    END ELSE IF (texttimer^.Elapsed > 400) THEN
      blurr32(testsurface,testsurface,Area(0,textypos,320,textypos+text_next_line));

    blurr32(testsurface,testsurface,Area(0,0,320,textypos));
    blurr32(testsurface,testsurface,Area(0,textypos+text_next_line,320,200));

    testconsole^.Copy(0,0,testsurface,Area(testsurface));
    testconsole^.Update;
  UNTIL (keypressed);

    // clear keyboard buffer
  WHILE (keypressed) DO readkey;

    // close Console and free that one, too
  testconsole^.Close;
  dispose(testconsole,Destroy);

  dispose(testsurface,Close);
  dispose(testfont,Close);

    // set textmode
  set_textmode;

  writeln('muu TrueColor library for TMT Pascal - Font example');
  writeln('Copyright (C) 1999 Tor Arne Gustad (anden@online.no)');
end.
