{
   muu TrueColor library for TMT Pascal - Plasma example
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

USES mArea,mFormat,mSurface,mConsole,mTarga,mc_VESA,mc_Wrap,Nova,memory,mfx32,CRT,Error,HTimer;

//{$DEFINE debug_wrapper}  // enable this ONLY for debugging

CONST xSize = 320;
      ySize = 200;

    // set textmode
PROCEDURE set_textmode; code;
ASM
  mov  ax,  $03
  int  $10
  ret
END;


VAR
  xtab, ytab:   ARRAY[0..xSize-1] OF Longint;
  suseg,c,cr:   dword := 0;
  cg,cb,x,y,f:  DWord := 0;
  testsurface:  Surface := NIL;
  testconsole:  Console := NIL;
  dimer:        Timer := NIL;

BEGIN
    // We will use the VBEConsole to display the image, therefore we overload
    // the testconsole type when creating it (creating means allocating)
    // (if debug_wrapper is defined, wrap VBEConsole to wrapper
  {$IFNDEF debug_wrapper}
    new(VBEConsole(testconsole),Create);
  {$ELSE}
    new(TTYConsole(testconsole),Create);
    DECLARE VAR deb: Console;
    BEGIN
      new(VBEConsole(deb),Create);
      testconsole^.Wrap(deb);
    END;
  {$ENDIF}

    // read options into Console, which will be used instead of the default
    // values if specified (here we load options from "mtc.cfg")
  testconsole^.getOptions(mtc_option_file);

    // options are set, and we can safely proceed opening our Console. Here
    // it's called "Image example", which is a suiting name:)
  testconsole^.Open('Plasma example',xsize,ysize,Format(Unknown));

    // check if an error occurred when opening, and exit with error if so
  IF (testconsole^.Error) THEN
    fail('Failed to create console');

  testsurface := Surface(xsize,ysize,Format(32));

  suseg := ofs(testsurface^.Data^);


  randomize;
  c := random(adeg);

  new(dimer,Create);
  dimer^.Start;
  REPEAT

    FOR x := 0 TO xSize-1 DO
      xtab[x] := (64+(sintab[(x shl 1-cr shl 2) AND adeg]+sintab[(x shl 2-cr shl 1) AND adeg]) shr 3) shl 16
                +(64+(costab[(x shl 2-cg shl 1) AND adeg]+costab[(x shl 3-cg shl 1) AND adeg]) shr 3) shl 8
                +(64+(sintab[(x shl 2-cb shl 3) AND adeg]+costab[(x shl 2-cb shl 2) AND adeg]) shr 3);

    FOR y := 0 TO ySize-1 DO
      ytab[y] := (64+integer(costab[(y shl 3+cr shl 1) AND adeg]+costab[(y shl 2-cr shl 3) AND adeg]) shr 3) shl 16
                +(64+integer(sintab[(y shl 1+cg shl 3) AND adeg]+sintab[(y shl 1-cg shl 2) AND adeg]) shr 3) shl 8
                +(64+integer(costab[(y shl 2+cb shl 1) AND adeg]+sintab[(y shl 2-cb shl 2) AND adeg]) shr 3);

    ASM
      mov  edi,  [suseg]
      mov  esi,  -(ySize*4)

    @@yhopp:
      mov  edx,  -(xSize*4)

      align 4
      align 4
    @@xhopp:
      mov  ecx,  dword ptr [xtab+edx+(xSize*4)]
      mov  eax,  dword ptr [ytab+esi+(ySize*4)]
      add  ecx,  eax

      mov  [edi],   ecx 

      add  edi,  4
      add  edx,  4
      jnz  @@xhopp

      add  esi,  4
      jnz  @@yhopp
    END;

    testconsole^.Copy(0,0,testsurface,Area(testsurface));
    testconsole^.Update;

    c +:= 40;
    cr +:= sintab[(c shr 3) AND adeg] shr 5;
    cg +:= sintab[((cr+1024) shr 3) AND adeg] shr 5;
    cb +:= costab[((cb+2048) shr 3) AND adeg] shr 5;

    f +:= 1;
  UNTIL (keypressed);
  dimer^.Stop;

    // clear keyboard buffer
  WHILE (keypressed) DO readkey;

    // close Console and free that one, too
  testconsole^.Close;
  dispose(testconsole,Destroy);

  dispose(testsurface,Close);

    // set textmode
  set_textmode;

  writeln('Frames per second: ', f/(dimer^.Elapsed/1000):0:3);writeln;

  writeln('muu TrueColor library for TMT Pascal - Plasma example');
  writeln('Copyright (C) 1999 Tor Arne Gustad (anden@online.no)');
END.
