{
   muu TrueColor library for TMT Pascal - Twirler w/Goraud shading
   Copyright (C) 2000 Arne Morten Kvarving (spiff@online.no)
    (modified for mTC output by Tor Arne Gustad (anden@online.no))

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

USES memory,error,mArea,mClip,mSurface,mConsole,mc_VESA,mFormat,Nova,CRT,mTarga,mFX32;

{$DEFINE GORAUD_SHADING}
//{$DEFINE PSYCHO_TWIRLER}

TYPE Twirler = ^TwirlerT;
     TwirlerT = OBJECT
                  public 
               
                   CONSTRUCTOR Create(tex:Surface;x_pos,spd,wdt,n_sides:longint);
                   DESTRUCTOR Destroy;

                   PROCEDURE Speed(new_sinspeed:longint);
                   PROCEDURE Position(new_startx:longint);
                   PROCEDURE Sides(new_numsides:longint);
                   PROCEDURE Width(new_m_width:dword);
                   PROCEDURE Texture(new_texture:Surface);
             
                   PROCEDURE Update;
                   PROCEDURE Draw(where:Surface);

                  private 

                    // variables
                   m_speed: longint;
                   m_count: longint;
                   m_position: longint;
                   m_sides: longint;
                   m_sidestep: longint;
                   m_texstep: longint;
                   m_texture: Surface;
                   m_width: dword;
                END;

CONSTRUCTOR TwirlerT.Create;
BEGIN
  m_count := 0;

  Position(x_pos);
  Speed(spd);
  Width(wdt);
  Sides(n_sides);
  Texture(tex);
END;

PROCEDURE TwirlerT.Speed;
  m_speed := new_sinspeed;

PROCEDURE TwirlerT.Position;
  m_position := new_startx;

PROCEDURE TwirlerT.Sides;
BEGIN
  m_sides := new_numsides;
  m_sidestep := 2048 div m_sides;
END;

PROCEDURE TwirlerT.Width;
  m_width := 8-new_m_width;

PROCEDURE TwirlerT.Texture;
  m_texture := new_texture;

PROCEDURE TwirlerT.Update;
  m_count +:= m_speed;

PROCEDURE TwirlerT.Draw;
VAR y: longint;
    tc: longint;
    x1,x2,x3,x4: longint;
    z1,z2,z3,z4: longint;
BEGIN
  FOR y := 0 TO (where^.Height-1) DO
  BEGIN
//    tc := m_count + (sintab[(y shl 2+m_count shl 1) AND adeg] shl 1);
    tc := m_count + (sintab[(y shl 2+m_count) AND adeg] shl 1)
                  - (costab[(y shl 1+m_count) AND adeg] shl 2);

{$IFDEF PSYCHO_TWIRLER}
    x1 := m_position+(sintab[(tc) AND adeg] shr m_width)+sintab[(m_count+(y+20) shl 1) AND adeg] shr 2;
    x2 := m_position+(costab[(tc+512+m_count shr 2) AND adeg] shr m_width)+sintab[(m_count+(y+12) shl 1) AND adeg] shr 2;
    x3 := m_position+(sintab[(tc+1024+m_count shr 2) AND adeg] shr m_width)+sintab[(m_count+(y+15) shl 1) AND adeg] shr 2;
    x4 := m_position+(costab[(tc+1536+m_count shr 2) AND adeg] shr m_width)+sintab[(m_count+(y+17) shl 1) AND adeg] shr 2;
{$ELSE}
    x1 := m_position+(sintab[(tc) AND adeg] shr m_width);
    x2 := m_position+(sintab[(tc+512) AND adeg] shr m_width);
    x3 := m_position+(sintab[(tc+1024) AND adeg] shr m_width);
    x4 := m_position+(sintab[(tc+1536) AND adeg] shr m_width);
{$ENDIF}


{$IFDEF GORAUD_SHADING}
        // shading variables
    z1 := (sintab[(tc shl 1) AND adeg] shr 2);
    z2 := (sintab[((tc shl 1)+1024) AND adeg] shr 2);
    z3 := (sintab[((tc shl 1)+256) AND adeg] shr 2);
    z4 := (sintab[((tc shl 1)+1280) AND adeg] shr 2);

    IF (x1-x4) > 0 THEN
      HLineG(y,x4,x1,192,255,z1,z3,m_texture,where);
    IF (x4-x3) > 0 THEN
      HLineG(y,x3,x4,128,192,z2,z4,m_texture,where); 
    IF (x3-x2) > 0 THEN
      HLineG(y,x2,x3, 64,127,z1,z3,m_texture,where);
    IF (x2-x1) > 0 THEN
      HLineG(y,x1,x2,  0, 63,z2,z4,m_texture,where);
{$ELSE}
    IF (x1-x4) > 0 THEN
      HLine(y,x4,x1,192,255,m_texture,where);
    IF (x4-x3) > 0 THEN
      HLine(y,x3,x4,128,192,m_texture,where);
    IF (x3-x2) > 0 THEN
      HLine(y,x2,x3, 64,127,m_texture,where);
    IF (x2-x1) > 0 THEN
      HLine(y,x1,x2,  0, 63,m_texture,where);
{$ENDIF}

  END;
END;

DESTRUCTOR TwirlerT.Destroy;
BEGIN
END;

VAR
  twrlr:    Twirler;
  con:      Console;
  texture:  Surface := loadTGA('data\map01.tga');
  virt:     Surface := Surface(320,200,format(BGR8888));
  c,x:      Longint := 0;

begin
    // try to open Console, read Options, exit if failure
  new(VBEConsole(con),Create);

    // get "common" options then try to get local options
  con^.getOptions('..\..\examples\'+mtc_option_file);
  con^.getOptions(mtc_option_file);

  con^.Open('mTC Twirler example',0,0,Format(Unknown));
  IF (con^.Error) THEN
    fail('Failed to create console');

  FOR c := 0 TO adeg DO
    sintab[c] := round(sin((c*6.284)/(adeg+2))*256);

  new(twrlr,Create(texture,160,10,37,4));
  REPEAT
    twrlr^.Update;
    virt^.Clear(Area(virt),0);
    twrlr^.Draw(virt);

    con^.Copy(0,0,virt,Area(virt));
    con^.Update;
  UNTIL (keypressed);

  dispose(twrlr,Destroy);
  dispose(texture,Close);
  dispose(virt,Close);

  con^.Close;
  dispose(con,Destroy);

  ASM mov ax,$3;int $10;END;

  writeln('muu TrueColor library for TMT Pascal - Twirler example');
  writeln('Copyright (C) 2000 Arne Morten Kvarving (spiff@online.no)');
end.
