    // x86 put pixels (not black)
PROCEDURE x86_put_bg_pixels(m_put:muuPutInterface); assembler;
  VAR tr_height:    dword := m_put^.Height;
      tr_width:     dword := m_put^.Width;
      tr_src:       pointer := m_put^.Source.Data;
      tr_dest:      pointer := m_put^.Destination.Data;
      tr_srcnl:     dword := m_put^.Source.Width;
      tr_destnl:    dword := m_put^.Destination.Width;
      bg:           dword := m_put^.Background;
ASM
    mov  edx,   tr_height
    mov  esi,   tr_src
    mov  edi,   tr_dest
    mov  ebx,   bg
    sub  esi,   4
    sub  edi,   4

  @@line_loop:
    mov  ecx,   tr_width

  @@pixel_loop:
    mov  eax,   [esi+ecx*4]
    cmp  eax,   ebx
    je   @@next_pixel
    mov  [edi+ecx*4], eax

  @@next_pixel:

    dec  ecx
    jnz  @@pixel_loop

    // skip to next line
    add  esi,   tr_srcnl
    add  edi,   tr_destnl
    dec  edx  
    jnz  @@line_loop
END;

    // x86 add pixels
PROCEDURE x86_add_pixels(m_put:muuPutInterface); assembler;
  VAR tr_height:    dword := m_put^.Height;
      tr_width:     dword := m_put^.Width;
      tr_src:       pointer := m_put^.Source.Data;
      tr_dest:      pointer := m_put^.Destination.Data;
      tr_srcnl:     dword := m_put^.Source.Width;
      tr_destnl:    dword := m_put^.Destination.Width;
ASM
    mov  edx,   tr_height
    mov  esi,   tr_src
    mov  edi,   tr_dest
    sub  esi,   4
    sub  edi,   4

  @@line_loop:
    mov  ecx,   tr_width

  @@pixel_loop:
    mov  eax,   [esi+ecx*4]
    mov  ebx,   [edi+ecx*4]

    // x86 pixel addition routine
  @@red:
    add  al,    bl
    jnc  @@green
    mov  al,    255

  @@green:
    add  ah,    bh
    jnc  @@blue
    mov  ah,    255

  @@blue:
    push ax
    shr  eax,   16
    shr  ebx,   16
    add  al,    bl
    jnc  @@put_pixel
    mov  al,    255

  @@put_pixel:
    shl  eax,   16
    pop  ax
    mov  [edi+ecx*4],  eax

    dec  ecx
    jnz  @@pixel_loop

  @@next_line:
    // skip to next line
    add  esi,   tr_srcnl
    add  edi,   tr_destnl
    dec  edx
    jnz  @@line_loop
END;

    // x86 subtract pixels
PROCEDURE x86_sub_pixels(m_put:muuPutInterface); assembler;
  VAR tr_height:    dword := m_put^.Height;
      tr_width:     dword := m_put^.Width;
      tr_src:       pointer := m_put^.Source.Data;
      tr_dest:      pointer := m_put^.Destination.Data;
      tr_srcnl:     dword := m_put^.Source.Width;
      tr_destnl:    dword := m_put^.Destination.Width;
ASM
    mov  edx,   tr_height
    mov  esi,   tr_src
    mov  edi,   tr_dest
    sub  esi,   4
    sub  edi,   4

  @@line_loop:
    mov  ecx,   tr_width

  @@pixel_loop:
    mov  ebx,   [esi+ecx*4]
    mov  eax,   [edi+ecx*4]

    // x86 pixel addition routine
  @@red:
    sub  al,    bl
    jnc  @@green
    xor  al,    al

  @@green:
    sub  ah,    bh
    jnc  @@blue
    xor  ah,    ah

  @@blue:
    push        ax
    shr  eax,   16
    shr  ebx,   16
    sub  al,    bl
    jnc  @@put_pixel
    xor  al,    al 

  @@put_pixel:
    shl  eax,   16
    pop  ax
    mov  [edi+ecx*4],  eax

    dec  ecx
    jnz  @@pixel_loop

  @@next_line:
    // skip to next line
    add  esi,   tr_srcnl
    add  edi,   tr_destnl
    dec  edx
    jnz  @@line_loop
END;

    // x86 add grade
PROCEDURE x86_add_grade(m_put:muuPutInterface); assembler;
  VAR tr_height:    dword := m_put^.Height;
      tr_width:     dword := m_put^.Width;
      tr_src:       pointer := m_put^.Source.Data;
      tr_dest:      pointer := m_put^.Destination.Data;
      tr_srcnl:     dword := m_put^.Source.Width;
      tr_destnl:    dword := m_put^.Destination.Width;
      g:            byte := m_put^.Grade;
ASM
    mov  edx,   tr_height
    mov  esi,   tr_src
    mov  edi,   tr_dest
    sub  esi,   4
    sub  edi,   4

  @@line_loop:
    mov  ecx,   tr_width

  @@pixel_loop:
    mov  eax,   [edi+ecx*4]
    mov  ebx,   [esi+ecx*4]

    // x86 pixel addition routine
  @@Blue:
    sub  bl,   g
    jnc  @@doBlue
    mov  bl,   0

  @@doBlue:
    add  al,   bl
    jnc  @@Green
    mov  al,   255

  @@Green:
    sub  bh,   g
    jnc  @@doGreen
    mov  bh,   0

  @@doGreen:
    add  ah,   bh
    jnc  @@Red
    mov  ah,   255

  @@Red:
    push ax
    shr  eax,  16
    shr  ebx,  16
    sub  bl,   g
    jnc  @@doRed
    mov  bl,   0

  @@doRed:
    add  al,   bl
    jnc  @@Finish
    mov  al,   255

  @@Finish:
    shl  eax,  16
    pop  ax

    mov  [edi+ecx*4],  eax

    dec  ecx
    jnz  @@pixel_loop

  @@next_line:
    // skip to next line
    add  esi,   tr_srcnl
    add  edi,   tr_destnl
    dec  edx
    jnz  @@line_loop
END;

