{
   muu Library for TMT Pascal - muu graphic effect base unit (32bpp code)
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

UNIT mFX32;

INTERFACE
{$R-,I+,Q-}

USES mClip,mArea,mFormat,mSurface,CPUInfo,Error;

CONST ADEG = 2047;
      M_PI = 3.141592654;

TYPE FX_CODE = (X86_CODE,MMX_CODE);

VAR sinTab:   ARRAY[0..ADEG] OF longint;
    cosTab:   ARRAY[0..ADEG] OF longint;

    // *** Horizontal line routines
PROCEDURE HLine(y,x1,x2,u1c,u2c:longint;texture,where:Surface);
PROCEDURE HLineG(y,x1,x2,u1c,u2c,s1c,s2c:longint;texture,where:Surface);

    // *** Surface copy routines
PROCEDURE SurfaceCopy_BG(xpos,ypos:longint;bg:dword;sprite,dest:Surface);
PROCEDURE SurfaceCopy_Add(xpos,ypos:longint;sprite,dest:Surface);
PROCEDURE SurfaceCopy_AddG(xpos,ypos:longint;grade:byte;sprite,dest:Surface);
PROCEDURE SurfaceCopy_Sub(xpos,ypos:longint;sprite,dest:Surface);
  OVERLOAD BGCopy = SurfaceCopy_BG;
  OVERLOAD AddCopy = SurfaceCopy_Add;
  OVERLOAD AddCopyG = SurfaceCopy_AddG;
  OVERLOAD SubCopy = SurfaceCopy_Sub;

PROCEDURE odSurfaceCopy_Add(dest,sprite:Surface);   // - To work with the
PROCEDURE odSurfaceCopy_Sub(dest,sprite:Surface);   // overloading parameters
  OVERLOAD +:= = odSurfaceCopy_Add;                 // - enables "s1 +:= s2"
  OVERLOAD -:= = odSurfaceCopy_Sub;                 // - enables "s1 -:= s2"


IMPLEMENTATION

    // ---------------
    //  copy routines
    // ----------

PROCEDURE HLine;
VAR hmultab:    pointer := where^.HMultab;
    uStep:      longint;
    uStepped:   longint;
    count:      longint := 0;
    tex:        pointer := texture^.Data;
    whe:        pointer := where^.Data;
BEGIN
  uStep := ((u2c-u1c) shl 9) div (x2-x1);
  uStepped := u1c shl 9;
  ASM
    mov  esi, tex
    mov  edi, whe
    mov  ebx, y
    mov  eax, hmultab
    add  edi, [eax+ebx*4]
    sal  ebx, 10
    add  esi, ebx
    mov  ecx, x2
    sub  ecx, x1
    mov  eax, x1
    sal  eax, 2
    add  edi, eax
    mov  ebx, uStepped
    mov  edx, uStep

  @@loopen:
    mov  eax, ebx
    sar  eax, 9
    mov  eax, [esi+eax*4]
    mov  [edi], eax
    add  ebx, edx
    add  edi, 4
    dec  ecx
    jnz  @@loopen
  END;
END;

PROCEDURE HLineG;
VAR hmultab:    pointer := where^.HMultab;
    uStep:      longint;
    uStepped:   longint;
    sStep:      longint;
    sStepped:   longint;
    count:      longint := 0;
    tex:        pointer := texture^.Data;
    whe:        pointer := where^.Data;
BEGIN
  uStep := ((u2c-u1c) shl 9) div (x2-x1);
  uStepped := u1c shl 9;

  IF (s1c < -255) THEN
    s1c := -255;
  IF (s2c < -255) THEN 
    s2c := -255;

  IF (s1c > 255) THEN
    s1c := 255;
  IF (s2c > 255) THEN
    s2c := 255;

  sStep := ((s2c-s1c) shl 9) div (x2-x1);
  sStepped := s1c shl 9;

  ASM
    mov  esi, tex
    mov  edi, whe
    mov  ebx, y
    mov  eax, hmultab
    add  edi, [eax+ebx*4]
    sal  ebx, 10
    add  esi, ebx
    mov  ecx, x2
    sub  ecx, x1
    mov  eax, x1
    sal  eax, 2
    add  edi, eax
    mov  ebx, uStepped
    mov  edx, sStepped

  @@loopen:
    mov  eax, ebx
    sar  eax, 9
    mov  eax, [esi+eax*4]

    push edx
    sar  edx, 9

  // pixel shading starts -->

    or   edx, edx
    js   @@shade_neg

    // ** add intensity
    add  al,    dl
    jnc  @@green_a
    mov  al,    255

  @@green_a:
    add  ah,    dl
    jnc  @@blue_a
    mov  ah,    255

  @@blue_a:
    push ax
    shr  eax,   16
    add  al,    dl
    jnc  @@put_pixel
    mov  al,    255

    jmp  @@put_pixel

    // ** subtract intensity
  @@shade_neg:
    neg  edx
    sub  al,    dl
    jnc  @@green_s
    xor  al,    al

  @@green_s:
    sub  ah,    dl
    jnc  @@blue_s
    xor  ah,    ah

  @@blue_s:
    push ax
    shr  eax,   16
    sub  al,    dl
    jnc  @@put_pixel
    xor  al,    al

  @@put_pixel:
    shl  eax,   16
    pop  ax

  // --> pixel shading ends
    pop  edx

    mov  [edi], eax

    add  ebx, uStep
    add  edx, sStep

    add  edi, 4
    dec  ecx
    jnz  @@loopen
  END;
END;


    // ---------------
    //  copy routines
    // ----------

TYPE muuData = RECORD
                 data:  pointer;
                 width: longint;
               END;

TYPE muuPutInterface = ^muuPutInterfaceT;
     muuPutInterfaceT = RECORD
                          width:        longint;
                          height:       longint;
                          source:       muuData;
                          destination:  muuData;
                          grade:        byte;
                          background:   dword;
                        END;

TYPE muuPutter = PROCEDURE(m_put:muuPutInterface);

{$I include\fx_x86.pas}
{$I include\fx_mmx.pas}
  
VAR put_bg_pixels: muuPutter;
    
    add_pixels:    muuPutter;
    sub_pixels:    muuPutter;
    
    add_pixels_grade:   muuPutter;
    sub_pixels_grade:   muuPutter;

FUNCTION sprite_clip(tx,ty:longint;m_sprite,m_dest:Surface): muuPutInterface;
VAR put_width,put_height:     longint := 0;
    scr_startx,scr_starty:    longint := 0;
    sprite_start:             dword := 0;
    dest_start:               dword := 0;
    m_area:                   Area := Area(m_sprite);
BEGIN
  IF (tx < 0) THEN
    m_area.Left -:= tx;

  IF (ty < 0) THEN
    m_area.Top -:= ty;

  scr_startX := (tx+abs(tx)) shr 1;
  scr_startY := (ty+abs(ty)) shr 1;

  m_area := Clip(m_area,Area(0,0,m_sprite^.Width,m_sprite^.Height));
  m_area := Clip(m_area,Area(0,0,m_area.Left+m_dest^.Width-scr_startx,m_area.Top+m_dest^.Height-scr_starty));

  put_width := m_area.Right-m_area.Left;
  put_height := m_area.Bottom-m_area.Top;

  sprite_start := (m_area.Left+m_area.Top*m_sprite^.Width)*Pitch(m_sprite);
  dest_start := (scr_startX+scr_startY*m_dest^.Width)*Pitch(m_dest^.sFormat);

  new(result);

  result^.width := put_width;
  result^.height := put_height;
  result^.source.data := ptr(ofs(m_sprite^.Data^)+sprite_start);
  result^.source.width := m_sprite^.width shl 2;
  result^.destination.data := ptr(ofs(m_dest^.Data^)+dest_start);
  result^.destination.width := m_dest^.width shl 2;
END;

PROCEDURE SurfaceCopy_BG;
  VAR pif:  muuPutInterface := NIL;
BEGIN
  IF ( (Depth(sprite) <> 32) OR (Depth(dest) <> 32) ) THEN
    fail('mPut: Only 32bpp surfaces supported');

  pif := sprite_clip(xpos,ypos,sprite,dest);

  pif^.background := bg;

  IF( (pif^.width > 0) AND (pif^.height > 0) ) THEN
    put_bg_pixels(pif);

  dispose(pif);
END;

PROCEDURE SurfaceCopy_Add;
  VAR pif:  muuPutInterface := NIL;
BEGIN
  IF ( (Depth(sprite) <> 32) OR (Depth(dest) <> 32) ) THEN
    fail('muuPutter: Only 32bpp surfaces supported');

  pif := sprite_clip(xpos,ypos,sprite,dest);

  IF( (pif^.width > 0) AND (pif^.height > 0) ) THEN
    add_pixels(pif);

  dispose(pif);
END;

PROCEDURE SurfaceCopy_Sub;
  VAR pif:  muuPutInterface := NIL;
BEGIN
  IF ( (Depth(sprite) <> 32) OR (Depth(dest) <> 32) ) THEN
    fail('muuPutter: Only 32bpp surfaces supported');

  pif := sprite_clip(xpos,ypos,sprite,dest);

  IF( (pif^.width > 0) AND (pif^.height > 0) ) THEN
    sub_pixels(pif);

  dispose(pif);
END;

PROCEDURE SurfaceCopy_AddG;
  VAR pif:  muuPutInterface := NIL;
BEGIN
  IF ( (Depth(sprite) <> 32) OR (Depth(dest) <> 32) ) THEN
    fail('muuPutter: Only 32bpp surfaces supported');

  pif := sprite_clip(xpos,ypos,sprite,dest);

  pif^.grade := grade;

  IF( (pif^.width > 0) AND (pif^.height > 0) ) THEN
    add_pixels_grade(pif);

  dispose(pif);
END;

PROCEDURE odSurfaceCopy_Add;
  SurfaceCopy_Add(0,0,sprite,dest);

PROCEDURE odSurfaceCopy_Sub;
  SurfaceCopy_Sub(0,0,sprite,dest);



    // ------------
    //  main code
    // --------

  VAR fxcode:   FX_CODE;
      c:        longint;

begin
    // generate sine&cosine tables
  FOR c := 0 TO adeg DO
  BEGIN
    sintab[c] := round(sin((c*M_PI*2)/(ADEG+2))*256);
    costab[c] := round(cos((c*M_PI*2)/(ADEG+2))*256);
  END;


    // determine CPU type
 fxcode := X86_CODE;

 IF ( cpu_info.mmx ) THEN
   fxcode := MMX_CODE
 ELSE IF (cpu_info.Family >= 3 ) THEN
   fxcode := X86_CODE;

    // set appropriate routines
 CASE (fxcode) OF

   X86_CODE:    BEGIN
                  put_bg_pixels := x86_put_bg_pixels;
                  
                  add_pixels := x86_add_pixels;
                  sub_pixels := x86_sub_pixels;

                  add_pixels_grade := x86_add_grade;
                END;


   MMX_CODE:    BEGIN
                  put_bg_pixels := x86_put_bg_pixels;
                  
                  add_pixels := mmx_add_pixels;
                  sub_pixels := mmx_sub_pixels;

                  add_pixels_grade := mmx_add_grade;
                END;
 END;


end.
