{
   muu TrueColor library for TMT Pascal - Surface unit
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

UNIT mSurface;

INTERFACE
{$R-,I+,Q-}

USES mArea,mClip,mFormat,CPUinfo,Nova,Memory,Error;

CONST Surface_default_Format =  BGR8888;

    // pword = pchar type with dword size
TYPE pDWORD_T = ARRAY[longint] OF dword;
     pDWORD =   ^pDWORD_T;

    // Surface object (fixed)
TYPE Surface  = ^SurfaceT;
     SurfaceT = OBJECT

                 public
                    // (width,height,format)
                  CONSTRUCTOR Open(w,h:dword;fmt:Format);
                  DESTRUCTOR Close;

                    // surface copy/clear
                  PROCEDURE Copy(tx,ty:longint;m_surface:Surface;m_area:Area);
                  PROCEDURE Clear(m_area:Area;m_color:dword);

                    // variables
                  Width,Height: dword;      // width and height
                  Data:         pointer;    // pointer to data
                  Size:         dword;      // data size
                  sFormat:      Format;     // pixel format ++
                  hmultab:      pDWORD;     // Horizontal multiplication table
                END;

// --------------------------------------------------------------------------
FUNCTION Surface_Open(w,h:dword;fmt:Format): Surface;
FUNCTION Surface_Default(w,h:dword): Surface;
FUNCTION Surface_Clone(m_surface:Surface): Surface;
FUNCTION Surface_Area_Open(m_area:Area;fmt:Format): Surface;
FUNCTION Surface_Clone_size(m_surface:Surface;fmt:Format): Surface;
  OVERLOAD Surface = Surface_Open;
  OVERLOAD Surface = Surface_Default;
  OVERLOAD Surface = Surface_Clone;
  OVERLOAD Surface = Surface_Area_Open;
  OVERLOAD Surface = Surface_Clone_size;

    // Surface information access
FUNCTION Surface_Pitch(m_surface:Surface): byte;
FUNCTION Surface_Depth(m_surface:Surface): byte;
FUNCTION Surface_Area(m_surface:Surface): Area;
FUNCTION Surface_Format(m_surface:Surface): Format;
  OVERLOAD Pitch = Surface_Pitch;
  OVERLOAD Depth = Surface_Depth;
  OVERLOAD Area = Surface_Area;
  OVERLOAD Format = Surface_Format;

IMPLEMENTATION

    // Surface initialization methods
FUNCTION Surface_Open;
  new(result,Open(w,h,fmt));

FUNCTION Surface_Default;
  new(result,Open(w,h,Format(Surface_default_Format)));

FUNCTION Surface_Clone;
  new(result,Open(m_surface^.Width,m_surface^.Height,m_surface^.sFormat));

FUNCTION Surface_Area_Open;
  new(result,Open(Width(m_area),Height(m_area),fmt));

FUNCTION Surface_Clone_size;
  new(result,Open(m_surface^.Width,m_surface^.Height,fmt));

    // Surface information aquirement
FUNCTION Surface_Pitch;
  result := Pitch(m_surface^.sFormat);

FUNCTION Surface_Depth;
  result := Depth(m_surface^.sFormat);

FUNCTION Surface_Area;
  result := Area(m_surface^.Width,m_surface^.Height);

FUNCTION Surface_Format;
  result := Format(m_surface^.sFormat);

// the Surface object 
// --------------------------------------------------------------------------
CONSTRUCTOR SurfaceT.Open;
  VAR c: dword;
BEGIN
  Width := w;
  Height := h;
  sFormat := Format(fmt);
  Size := Width*Height*Pitch(sFormat);
  getmem(Data,Size);
  memset(Data,Size,0);

  // create hmultab
  getmem(hmultab,Height*4);

  FOR c := 0 TO (Height-1) DO
    hmultab^[c] := c*Width*Pitch(sFormat);
END;

PROCEDURE SurfaceT.Clear;
  VAR clearer:    Nova_Clearer := NIL;
BEGIN
  clearer := Nova_getClearer(cpu_info,Pitch(self.sFormat));

  IF (@clearer = NIL) THEN
    fail('Unsupported clearer depth.');

  DECLARE VAR put_width,put_height: dword;
              data_start:           dword;
              pixel_clear:          dword;
  BEGIN
    m_area := Clip(m_area,Area(0,0,self.Width,self.Height));
    put_width := m_area.Right-m_area.Left;
    put_height := m_area.Bottom-m_area.Top;

    IF (put_width <= 0) OR
       (put_height <= 0) THEN
         Exit;

    data_start := ((m_area.Top*self.Width)+m_area.Left)*Pitch(sFormat);

    CASE Depth(sformat) OF
      16:   pixel_clear := Pack16(m_color)+Pack16(m_color) shl 16;
      15:   pixel_clear := Pack15(m_color)+Pack15(m_color) shl 16;
       8:   pixel_clear := Pack8(m_color)+(Pack8(m_color) shl 8)+
                           (Pack8(m_color) shl 16)+(Pack8(m_color) shl 24);
     ELSE
       pixel_clear := m_color;

    END;

    clearer(ptr(ofs(self.Data^)+data_start),
            pixel_clear,
            put_width,
            put_height,
            (self.Width-put_width)*Pitch(sFormat));
  END; 
END;

PROCEDURE SurfaceT.Copy;
  VAR convert:    Nova_converter;
BEGIN
  convert := Nova_getConverter(cpu_info,m_surface^.sFormat.pixelformat,sFormat.pixelformat);

    // special case, program should be exited anyway
  IF (@convert = NIL) THEN
    fail('Unsupported color conversion.');

  DECLARE VAR put_width,put_height:     longint;
              scr_startx,scr_starty:    longint;
              data_start:               dword;
  BEGIN
    IF (tx < 0) THEN
      m_area.Left -:= tx;

    IF (ty < 0) THEN
      m_area.Top -:= ty;

    scr_startX := (tx+abs(tx)) shr 1;
    scr_startY := (ty+abs(ty)) shr 1;

    m_area := Clip(m_area,Area(0,0,m_surface^.Width,m_surface^.Height));
    m_area := Clip(m_area,Area(0,0,m_area.Left+self.Width-scr_startx,m_area.Top+self.Height-scr_starty));

    put_width := m_area.Right-m_area.Left;
    put_height := m_area.Bottom-m_area.Top;

    IF (put_width <= 0) OR
       (put_height <= 0) THEN
         Exit;

    data_start := (m_area.Left+m_area.Top*m_surface^.Width)*Pitch(m_surface);

    IF (m_surface^.sFormat.Pixelformat = VPAL) THEN
      Nova_setVPal(m_surface^.sFormat.Palette);

    convert(ptr(ofs(m_surface^.Data^)+data_start),
            ptr(ofs(self.Data^)+(scr_startX+scr_startY*self.Width)*Pitch(self.sFormat)),
            put_width,
            put_height,
            (m_surface^.Width-put_width)*Pitch(m_surface),
            (self.Width-put_width)*Pitch(self.sFormat));
  END; 
END;

DESTRUCTOR SurfaceT.Close;
BEGIN
  freemem(hmultab,Height shl 2);
  freemem(Data,Size);
END;


end.
