{
   muu TrueColor library for TMT Pascal - CPU information unit
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.       
}

UNIT cpuinfo;

INTERFACE
{$R-,I+,Q-}

USES Error;


// --------------------------------------------------------------------------
TYPE cpuinfoT = RECORD
                  Family:       byte;       // family (e.g 3=386,4=486...)
                  FPU,MMX,Dual: boolean;    // extensions
                  TSC,CX8,CMOV: boolean;    // special instructions
                  MTRR,MSR:     boolean;    // special features
                END;

    // new interface (global)
VAR cpu_info:   cpuinfoT;

    // old interface
PROCEDURE getcpuinfo(VAR gcpu:cpuinfoT);    // retrieve cpu information


// --------------------------------------------------------------------------
IMPLEMENTATION

CONST _cpuflags_align = $40000;     // alignment check flag
      _cpuflags_cpuid = $200000;    // cpuid (instruction)
      _cpuid_FPU =      $1;         // fpu
      _cpuid_TSC =      $8;         // time stamp counter
      _cpuid_MSR =      $20;        // rdmsr/wrmsr support
      _cpuid_CX8 =      $80;        // cmpxchg8b (instruction)
      _cpuid_MTRR =     $1000;      // memory type range register
      _cpuid_CMOV =     $4000;      // cmovcc (instructions)
      _cpuid_MMX =      $800000;    // mmx
      _msw_FPU  =       $2;         // generic fpu

{ this detection works like this:
   (0. detection of 8086/8088 and 286, not needed and doesn't work in p.mode)
    1. check for a 386 by setting the flags 12-14 (only allowed by 386s)
    2. check if flag 22 is set (extended) which is the cpuid feature; old 486
       processors don't have this feature
    3. use the cpuid instruction to determine processor type and features
    4. find generic floating point unit by checking flag 2 of machine status
       word (MSW), aquired by instruction smsw (store machine status word)
    }
PROCEDURE getcpuinfo(VAR gcpu:cpuinfoT);
  VAR tcpu: cpuinfoT;
BEGIN
  fillchar(tcpu,sizeof(cpuinfoT),0);
  ASM
    // check if cpu is a 386
    pushfd
    pop  eax
    or   eax,  _cpuflags_align    // set the alignment check flag
    push eax
    popfd
    pushfd
    pop  eax
    and  eax,  _cpuflags_align    // if this is off, cpu is a 386
    jnz  @@check486
    jmp  @@checkFPU

    // detect whether `cpuid' is supported or not
  @@check486:
    inc  byte [tcpu.Family]
    pushfd
    pop  eax
    or   eax,  _cpuflags_cpuid    // set the cpuid flag
    push eax
    popfd
    pushfd
    pop  eax
    and  eax,  _cpuflags_cpuid    // if turned off, cpu is an old-type 486
    jnz  @@cpuid
    jmp  @@checkFPU

    // get cpu information with the `cpuid' instruction
  @@cpuid:
    xor  eax,  eax
    dw   $A20F  // cpuid (swapped)
    cmp  eax,  1
    jb   @@checkFPU

    mov  eax,  1
    dw   $A20F  // cpuid (swapped)

    push eax
    shr  eax,  8            // bits 8-11 holds the cpu family
    and  eax,  $0F
    mov  [tcpu.Family],  al // get cpu family (486+)
    pop  eax

    shr  eax,  12           // bits 12-15 holds some "reserved" information
    and  eax,  $0F
    cmp  eax,  2
    setz byte  [tcpu.Dual]  // is this a dual CPU board? (PPro/P2)

    test edx,  _cpuid_FPU
    setnz byte [tcpu.FPU]   // is an FPU present? (486DX,Pentium,PPro,P2)
    test edx,  _cpuid_TSC
    setnz byte [tcpu.TSC]   // is `rdtsc' supported? (Pentium+)
    test edx,  _cpuid_MSR
    setnz byte [tcpu.MSR]   // is `rdmsr' and `wrmsr' supported (Pentium+)
    test edx,  _cpuid_CX8
    setnz byte [tcpu.CX8]   // is `cmpxchg8b' supported? (Pentium+)
    test edx,  _cpuid_MTRR
    setnz byte [tcpu.MTRR]  // is memory type range register supported (PentiumPro+)
    test edx,  _cpuid_CMOV
    setnz byte [tcpu.CMOV]  // is conditional move supported (PPro/P2)
    test edx,  _cpuid_MMX
    setnz byte [tcpu.MMX]   // is MMX instructions supported (P-MMX,P2)
    jmp  @@done

    // generic FPU detection routine
  @@checkFPU:
    smsw ax     // load the machine status word in ax
    test ax,   _msw_FPU
    setnz byte [tcpu.FPU]

  @@done:
  END;

  move(tcpu,gcpu,sizeof(cpuinfoT)); // move information to getcpu record
END;


//---------------------------------------------------------------------------
begin
    // get global information
  getcpuinfo(cpu_info);
end.
