{
   muu TrueColor library for TMT Pascal - VESA VBE Core unit
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.       
}

UNIT VBECore;

INTERFACE
{$R-,I+,Q-}

USES DPMI;

    // VESA return value if function is successful
CONST VBEsuccess =  $004F;

    // VESA window modes (used in VBE_setwindow and VBE_getwindow)
TYPE VBEwindowT =   (WindowA,WindowB);

    // VESA states (used in VBE_getstate and VBE_setstate)
    // (can be used in conjunction, e.g [state1] AND [state2])
CONST VBEstateHWC =     $01;    // ?? state
      VBEstateBIOS =    $02;    // BIOS state
      VBEstateDAC =     $04;    // DAC state
      VBEstateREG =     $08;    // register state
      VBEstateALL =     $0F;

    // VESA functions
FUNCTION VBE_getinfo(buf:word): shortint;
FUNCTION VBE_getmodeinfo(buf,mode:word): shortint;
    // --
FUNCTION VBE_setmode(mode:word): shortint;
FUNCTION VBE_getmode(VAR mode:word): shortint;
    // --
FUNCTION VBE_getstatebuffersize(VAR size:word): shortint;
FUNCTION VBE_getstate(buf,stat:word): shortint;
FUNCTION VBE_setstate(buf,stat:word): shortint;
    // --
FUNCTION VBE_setwindow(wn_N:VBEwindowT;wndw:word): shortint;
FUNCTION VBE_getwindow(wn_N:VBEwindowT;VAR wndw:word): shortint;
    // --
FUNCTION VBE_setdisplaystart(xpan,ypan:integer): shortint;
FUNCTION VBE_getdisplaystart(VAR xpan,ypan:integer): shortint;
    // --
FUNCTION VBE_setDACwidth(width:byte): shortint;
FUNCTION VBE_getDACwidth(VAR width:byte): shortint;
    // --
FUNCTION VBE_setpalette(buf,first,count:word): shortint;
FUNCTION VBE_getpalette(buf,first,count:word): shortint;
    // --
FUNCTION VBE_getprotectedmodeinterface(VAR segm,ofst,len:word): shortint;


IMPLEMENTATION

// --------------------------------------------------------------------------
{ VBE_getinfo(buf:word): shortint;
    * Function 0x4F00 / 0x10:    Return VBE controller information
    * Input:    buf -   buffer located in lower 640k 
    }
FUNCTION VBE_getinfo(buf:word): shortint;
  VAR regs: tRMregs;
BEGIN
  clearRMRegs(regs);
  regs.ax := $4F00;
  regs.es := buf;
  realmodeint($10,regs);

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

{ VBE_getmodeinfo(buf,mode:word): shortint;
    * Function 0x4F01 / 0x10:    Get VBE mode information
    * Input:    buf -   buffer located in lower 640k
                mode -  mode number
    }
FUNCTION VBE_getmodeinfo(buf,mode:word): shortint;
  VAR regs: tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F01;
  regs.cx := mode;
  regs.es := buf;
  realmodeint($10,regs);

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

// --------------------------------------------------------------------------
{ VBE_setmode(mode:word): shortint;
    * Function 0x4F02 / 0x10:    Set VBE mode
    * Input:    mode -  mode number
    }
FUNCTION VBE_setmode(mode:word): shortint;
  VAR regs: tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F02;
  regs.bx := mode;
  realmodeint($10,regs);

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

{ VBE_getmode(VAR mode:word): shortint;
    * Function 0x4F03 / 0x10:    Return current VBE mode
    * Output:   mode -  mode number
    }
FUNCTION VBE_getmode(VAR mode:word): shortint;
  VAR regs: tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F03;
  realmodeint($10,regs);

  mode := regs.bx;

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

// --------------------------------------------------------------------------
{ VBE_getstatebuffersize(VAR size:word): shortint;
    * Function 0x4F04 / 0x10:    Save/Restore state
    * Output:   size -  size in 64 byte blocks required to hold state
    }
FUNCTION VBE_getstatebuffersize(VAR size:word): shortint;
  VAR regs: tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F04;
  regs.dl := $00;   // Return save/restore state buffer size
  realmodeint($10,regs);

  size := regs.bx;

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

{ VBE_getstate(stat:VBEstateT;buf:word): shortint;
    * Function 0x4F04 / 0x10:    Save/Restore state
    * Input:    stat -  states to save 
                buf -   buffer to save state in
    }
FUNCTION VBE_getstate(buf,stat:word): shortint;
  VAR regs: tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F04;
  regs.dl := $01;   // Save state
  regs.cx := stat;
  regs.es := buf;
  realmodeint($10,regs);

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

{ VBE_setstate(stat:VBEstateT;buf:word): shortint;
    * Function 0x4F04 / 0x10:    Save/Restore state
    * Input:    stat -  states to restore (must be equal to saved states!)
                buf -   buffer to restore state from
    }
FUNCTION VBE_setstate(buf,stat:word): shortint;
  VAR regs: tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F04;
  regs.dl := $02;   // Restore state
  regs.cx := stat;
  regs.es := buf;
  realmodeint($10,regs);

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

// --------------------------------------------------------------------------
{ VBE_setwindow(wn_N:VBEwindowT;wndw:word): shortint;
    * Function 0x4F05 / 0x10:    Display window control
    * Input:    wn_N -  window number (A/B)
                wndw -  window number in video memory (in window)
    }
FUNCTION VBE_setwindow(wn_N:VBEwindowT;wndw:word): shortint;
  VAR regs: tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F05;
  regs.bh := $00;       // Set memory window
  regs.bl := byte(wn_N);    // 0 or 1
  regs.dx := wndw;
  realmodeint($10,regs);

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

{ VBE_getwindow(wn_N:VBEwindowT;VAR wndw:word): shortint;
    * Function 0x4F05 / 0x10:    Display window control
    * Input:    wn_N -  window number (A/B)
    * Output:   wndw -  window number in video memory (in window)
    }
FUNCTION VBE_getwindow(wn_N:VBEwindowT;VAR wndw:word): shortint;
  VAR regs: tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F05;
  regs.bh := $01;       // Get memory window
  regs.bl := byte(wn_N);    // 0 or 1
  realmodeint($10,regs);

  wndw := regs.dx;

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

// --------------------------------------------------------------------------
{ VBE_setdisplaystart(xpan,ypan:integer): shortint;
    * Function 0x4F07 / 0x10:    Set/get display start
    * Input:    xpan -  horizontal panning (x-axis)
                ypan -  vertical panning (y-axis)
    }
FUNCTION VBE_setdisplaystart(xpan,ypan:integer): shortint;
  VAR regs:tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F07;
  regs.bl := $00;   // Get display start
  regs.cx := xpan;
  regs.dx := ypan;
  realmodeint($10,regs);

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

{ VBE_getdisplaystart(VAR xpan,ypan:integer): shortint;
    * Function 0x4F07 / 0x10:    Set/get display start
    * Output:   xpan -  horizontal panning (x-axis)
                ypan -  vertical panning (y-axis)
    }
FUNCTION VBE_getdisplaystart(VAR xpan,ypan:integer): shortint;
  VAR regs:tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F07;
  regs.bl := $01;   // Get display start
  realmodeint($10,regs);

  xpan := regs.cx;
  ypan := regs.dx;

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

// --------------------------------------------------------------------------
{ VBE_setDACwidth(width:byte): shortint;
    * Function 0x4F08 / 0x10:    Set/get DAC palette format
    * Input:   width - DAC width (in bits)
    }
FUNCTION VBE_setDACwidth(width:byte): shortint;
  VAR regs:tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F08;
  regs.bl := $00;
  regs.bh := width;
  realmodeint($10,regs);

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

{ VBE_getDACwidth(VAR width:byte): shortint;
    * Function 0x4F08 / 0x10:    Set/get DAC palette format
    * Output:   width - DAC width (in bits)
    }
FUNCTION VBE_getDACwidth(VAR width:byte): shortint;
  VAR regs:tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F08;
  regs.bl := $01;
  realmodeint($10,regs);

  width := regs.bh;

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

// --------------------------------------------------------------------------
{ VBE_setpalette(buf,first,count:word): shortint;
    * Function 0x4F09 / 0x10:    Set/get palette data
    * Input:    buf -   buffer located in lower 640k
                first - first palette entry to be changed
                count - number of palette entrys to change
    }
FUNCTION VBE_setpalette(buf,first,count:word): shortint;
  VAR regs:tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F09;
  regs.bl := $00;
  regs.cx := count;
  regs.dx := first;
  regs.es := buf;
  realmodeint($10,regs);

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

{ VBE_getpalette(buf,first,count:word): shortint;
    * Function 0x4F09 / 0x10:    Set/get palette data
    * Input:    buf -   buffer located in lower 640k
                first - first palette entry to be aquired
                count - number of palette entrys to aquired
    }
FUNCTION VBE_getpalette(buf,first,count:word): shortint;
  VAR regs:tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F09;
  regs.bl := $01;
  regs.cx := count;
  regs.dx := first;
  regs.es := buf;
  realmodeint($10,regs);

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;

// --------------------------------------------------------------------------
{ VBE_getprotectedmodeinterface(VAR segm,ofst,len:word): shortint;
    * Function 0x4F0A / 0x10:    Return VBE Protected Mode interface
    * Output:   segm -  segment of table
                ofst -  offset of table
                len -   length of table including protected mode code
    }
FUNCTION VBE_getprotectedmodeinterface(VAR segm,ofst,len:word): shortint;
  VAR regs:tRMregs;
BEGIN
  clearRMregs(regs);
  regs.ax := $4F0A;
  regs.bl := $00;
  realmodeint($10,regs);

  segm := regs.es;
  ofst := regs.di;
  len := regs.cx;

  IF (regs.ax <> VBEsuccess) THEN
    result := -1
  ELSE
    result := 0;
END;


// --------------------------------------------------------------------------
end.
